package chess.mobile.browsegames;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.PopupMenu;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.browsepgnfile.BrowsePgnActivity;
import chess.mobile.game.GameActivity;
import chess.mobile.model.SavedGameFileMeta;
import chess.mobile.util.OnItemClickListener;

/**
 * Created by Stanisław Kabaciński.
 */

public class BrowseFragment extends Fragment implements BrowseGamesContract.View, OnItemClickListener<SavedGameFileMeta> {

    private static final int REQUEST_LOAD_FILE = 1234;

    @BindView(R.id.recycler_view) RecyclerView recyclerView;

    private BrowseGamesContract.Presenter presenter;
    private BrowseGamesAdapter adapter;

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        adapter = new BrowseGamesAdapter(this, new ArrayList<>(0));
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_browse_games, container, false);
        ButterKnife.bind(this, view);

        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        recyclerView.setAdapter(adapter);
        return view;
    }

    @Override
    public void showSavedGames(List<SavedGameFileMeta> savedGames) {
        adapter.setDataSet(savedGames);
        adapter.notifyDataSetChanged();
    }

    @Override
    public void cannotRemoveGame() {
        showInfo(getString(R.string.cannot_remove_game));
    }

    @Override
    public void notifyGameRemoved(int position) {
        adapter.notifyItemRemoved(position);
        showInfo(getString(R.string.game_removed));
    }

    @Override
    public void openFile(String path) {
        Intent intent = new Intent(getActivity(), BrowsePgnActivity.class);
        intent.putExtra(BrowsePgnActivity.KEY_FILE_PATH, path);
        startActivityForResult(intent, REQUEST_LOAD_FILE);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK && requestCode == REQUEST_LOAD_FILE) {
            Intent intent = new Intent();
            intent.putExtra(GameActivity.KEY_GAME_ID, data.getIntExtra(GameActivity.KEY_GAME_ID, -1));
            getActivity().setResult(Activity.RESULT_OK, intent);
            getActivity().finish();
        }
    }

    @Override
    public void setPresenter(BrowseGamesContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @Override
    public void onItemClick(View v, int position, SavedGameFileMeta object) {
        if (v.getId() == R.id.button) {
            PopupMenu popup = new PopupMenu(getContext(), v);

            popup.getMenuInflater().inflate(R.menu.popup_browse_games, popup.getMenu());

            popup.setOnMenuItemClickListener(item -> {
                switch (item.getItemId()) {
                    case R.id.delete:
                        presenter.removeFile(position);
                        return true;
                    default:
                        return false;
                }
            });
            popup.show();
        } else {
            presenter.onGameSelected(object);
        }
    }

    private void showInfo(String info) {
        Toast.makeText(getContext(), info, Toast.LENGTH_LONG).show();
    }
}
