package chess.mobile.browsegames;

import java.util.Collections;
import java.util.List;

import chess.mobile.dao.SavedGamesDAO;
import chess.mobile.model.SavedGameFileMeta;

/**
 * Created by Stanisław Kabaciński.
 */

public class BrowsePresenter implements BrowseGamesContract.Presenter {

    private BrowseGamesContract.View view;
    private SavedGamesDAO savedGamesDAO;
    private List<SavedGameFileMeta> savedGames;

    public BrowsePresenter(BrowseGamesContract.View view, SavedGamesDAO savedGamesDAO) {
        this.view = view;
        this.savedGamesDAO = savedGamesDAO;
        this.view.setPresenter(this);
    }

    @Override
    public void start() {
        savedGamesDAO.getPgnFilesList(new SavedGamesDAO.Callback<List<SavedGameFileMeta>>() {
            @Override
            public void onSuccess(List<SavedGameFileMeta> data) {
                Collections.sort(data, (o1, o2) -> o2.getLastModified().compareTo(o1.getLastModified()));
                savedGames = data;
                view.showSavedGames(data);
            }

            @Override
            public void onFailure(Exception e) {

            }
        });
    }

    @Override
    public void onGameSelected(SavedGameFileMeta object) {
        view.openFile(object.getPath());
    }

    @Override
    public void removeFile(int position) {
        savedGamesDAO.removeFile(savedGames.get(position).getPath(), new SavedGamesDAO.Callback<String>() {
            @Override
            public void onSuccess(String data) {
                savedGames.remove(position);
                view.notifyGameRemoved(position);
            }

            @Override
            public void onFailure(Exception e) {
                view.cannotRemoveGame();
            }
        });
    }
}
