package chess.mobile.browsepgnfile;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.chessgame.pgn.PGNGame;
import chess.mobile.game.GameActivity;
import chess.mobile.util.OnItemClickListener;

/**
 * Created 07.07.2017.
 */

public class BrowsePgnFragment extends Fragment implements BrowsePgnContract.View {

    @BindView(R.id.recycler_view) RecyclerView recyclerView;
    private BrowsePgnContract.Presenter presenter;
    private List<PGNGame> games;
    private ProgressDialog progressDialog;

    @Override
    public void setPresenter(BrowsePgnContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @Override
    public void onStart() {
        super.onStart();
        presenter.start();
    }

    @Override
    public void onPause() {
        super.onPause();
        presenter.stop();
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View v = inflater.inflate(R.layout.fragment_browse_pgn, container, false);
        ButterKnife.bind(this, v);
        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        recyclerView.setAdapter(new BrowsePgnAdapter());
        return v;
    }

    @Override
    public void showGames(List<PGNGame> gameList) {
        this.games = gameList;
        recyclerView.getAdapter().notifyDataSetChanged();
    }

    @Override
    public void showGame(int id) {
        Intent intent = new Intent();
        intent.putExtra(GameActivity.KEY_GAME_ID, id);
        getActivity().setResult(Activity.RESULT_OK, intent);
        getActivity().finish();
    }

    @Override
    public void showLoading(boolean show) {
        if (show) {
            progressDialog = new ProgressDialog(getContext());
            progressDialog.setCancelable(false);
            progressDialog.setTitle(null);
            progressDialog.setMessage(getString(R.string.please_wait));
            progressDialog.setIndeterminate(true);
            progressDialog.show();
        } else {
            if (progressDialog != null) {
                progressDialog.dismiss();
            }
        }
    }

    private class BrowsePgnAdapter extends RecyclerView.Adapter<PgnGameViewHolder> implements OnItemClickListener<PGNGame> {

        @Override
        public PgnGameViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            return new PgnGameViewHolder(inflater.inflate(R.layout.item_pgn_game, parent, false), this);
        }

        @Override
        public void onBindViewHolder(PgnGameViewHolder holder, int position) {
            holder.bind(games.get(holder.getAdapterPosition()));
        }

        @Override
        public int getItemCount() {
            return games == null ? 0 : games.size();
        }

        @Override
        public void onItemClick(View v, int position, PGNGame game) {
            presenter.loadGame(game);
        }
    }


}
