package chess.mobile.dao;

import chess.mobile.model.Engine;
import chess.mobile.model.EnginesList;
import chess.mobile.model.Position;
import chess.mobile.model.StartEngine;
import chess.mobile.rest.ApiCallback;
import chess.mobile.rest.EngineService;
import chess.mobile.rest.Response;
import chess.mobile.rest.ResponseListener;
import chess.mobile.rest.ServiceGenerator;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineDAOImpl implements EngineDAO {

    private UserSettingsDAO userSettingsDAO;

    public EngineDAOImpl(UserSettingsDAO userSettingsDAO) {
        this.userSettingsDAO = userSettingsDAO;
    }

    @Override
    public void getEnginesList(ResponseListener<Engine[]> callback) {
        EngineService engineService = ServiceGenerator.createService(EngineService.class, userSettingsDAO.getAuthToken());
        engineService.getEnginesList().enqueue(new ApiCallback<>(new ResponseListener<EnginesList>() {
            @Override
            public void onRequestFailure(Exception e) {
                callback.onRequestFailure(e);
            }

            @Override
            public void onRequestSuccess(EnginesList response) {
                callback.onRequestSuccess(response.getEngines());
            }
        }));
    }

    @Override
    public void startEngine(String engineName, ResponseListener<Response> callback) {
        EngineService engineService = ServiceGenerator.createService(EngineService.class, userSettingsDAO.getAuthToken());
        StartEngine startEngine = new StartEngine();
        startEngine.engine = engineName;
        engineService.startEngine(startEngine).enqueue(new ApiCallback<>(callback));
    }

    @Override
    public void stopEngine(ResponseListener<Response> callback) {
        EngineService engineService = ServiceGenerator.createService(EngineService.class, userSettingsDAO.getAuthToken());
        engineService.stopEngine().enqueue(new ApiCallback<>(callback));
    }

    @Override
    public void setPosition(String moves, ResponseListener<Response> callback) {
        Position position = new Position();
        position.moves = moves;
        EngineService engineService = ServiceGenerator.createService(EngineService.class, userSettingsDAO.getAuthToken());
        engineService.setPosition(position).enqueue(new ApiCallback<>(callback));
    }

}
