package chess.mobile.dao;

import android.os.Environment;
import android.util.Log;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import chess.mobile.chessgame.pgn.PGNGame;
import chess.mobile.chessgame.pgn.MoveTextReader;
import chess.mobile.chessgame.pgn.PGNReader;
import chess.mobile.chessgame.pgn.PGNWriter;
import chess.mobile.model.SavedGameFileMeta;

/**
 * Created by Stanisław Kabaciński.
 */

public class SavedGamesDAOImpl implements SavedGamesDAO {
    private static final String STORAGE_DIR = "pgn_games";
    private static final String TAG = SavedGamesDAOImpl.class.getSimpleName();

    private List<PGNGame> games;

    private static class Handler {
        static final SavedGamesDAOImpl INSTANCE = new SavedGamesDAOImpl();
    }

    public static SavedGamesDAO getInstance() {
        return Handler.INSTANCE;
    }

    private SavedGamesDAOImpl() {
    }

    @Override
    public void saveChessGame(String name, PGNGame game, Callback<String> callback) {
        PGNWriter writer = new PGNWriter();
        String gameString = writer.write(game.getEntities(), game.getMeta());

        File root = getGamesStorageDir();
        File gameFile = new File(root, name + ".pgn");

        try {
            FileOutputStream f = new FileOutputStream(gameFile);
            f.write(gameString.getBytes());
            f.close();
            callback.onSuccess(gameFile.getPath());
        } catch (IOException e) {
            e.printStackTrace();
            callback.onFailure(e);
        }
    }



    @Override
    public void getPgnFilesList(Callback<List<SavedGameFileMeta>> callback) {
        File root = getGamesStorageDir();
        File[] files = root.listFiles();
        List<SavedGameFileMeta> list = new ArrayList<>();
        for (File f: files) {
            SavedGameFileMeta s = new SavedGameFileMeta();
            if (!f.isDirectory() && !f.isHidden()) {
                s.setPath(f.getAbsolutePath());
                s.setName(f.getName());
                s.setLastModification(new Date(f.lastModified()));
                list.add(s);
            }
        }
        callback.onSuccess(list);
    }

    @Override
    public PGNGame getTemporalGame() {
        List<PGNGame> games = getGames(getGamesStorageDir() + "/.tmp.pgn");
        if (games == null || games.size() == 0) {
            return null;
        }
        return getGame(games.get(0).getId());
    }

    @Override
    public void removeTemporalGame() {
        removeFile(getGamesStorageDir() + "/.tmp.pgn", new Callback<String>() {
            @Override
            public void onSuccess(String data) {
            }

            @Override
            public void onFailure(Exception e) {

            }
        });
    }

    @Override
    public PGNGame getGame(int id) {
        if (games == null) {
            return null;
        }
        for (PGNGame game : games) {
            if (game.getId() == id) {
                return game;
            }
        }
        return null;
    }

    @Override
    public List<PGNGame> getGames(String path) {
        List<PGNGame> games;
        Reader reader;
        try {
            File file = new File(path);
            reader = new BufferedReader(new FileReader(file));
            PGNReader pgnReader = new PGNReader();
            games = pgnReader.read(reader);
            reader.close();
        } catch (IOException e) {
            return null;
        }
        this.games = games;
        return games;
    }


    @Override
    public void removeFile(String path, Callback<String> callback) {
        File file = new File(path);
        if (file.delete()) {
            callback.onSuccess(path);
        } else {
            callback.onFailure(new Exception());
        }
    }


    private File getGamesStorageDir() {
        File file = new File(Environment.getExternalStorageDirectory(), STORAGE_DIR);
        if (!file.mkdirs()) {
            Log.i(TAG, "Directory already exists");
        }
        return file;
    }

    private static String convertStreamToString(InputStream is) throws Exception {
        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        StringBuilder sb = new StringBuilder();
        String line;
        while ((line = reader.readLine()) != null) {
            sb.append(line).append("\n");
        }
        reader.close();
        return sb.toString();
    }

    private static String getStringFromFile (String filePath) throws Exception {
        File fl = new File(filePath);
        FileInputStream fin = new FileInputStream(fl);
        String ret = convertStreamToString(fin);
        fin.close();
        return ret;
    }



}
