package chess.mobile.dao.manager;

import android.util.Log;

import java.util.ArrayList;
import java.util.List;

import chess.mobile.Uci;
import chess.mobile.model.engineoptions.EngineOption;
import chess.mobile.model.engineoptions.EngineOptionFactory;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineManager {
    private static final String TAG = EngineManager.class.getSimpleName();
    private static volatile EngineManager INSTANCE = new EngineManager();

    public static EngineManager getInstance() {
        return INSTANCE;
    }

    private List<EngineEventListener> listeners = new ArrayList<>();
    private List<EngineOption> engineOptions = new ArrayList<>();
    private String engineName;
    private boolean ready = false;

    private EngineManager() {}

    public void interceptEngineOutput(String line) {
        if (line.contains(Uci.UCI_OK)) {
            ready = true;
            for (EngineEventListener l : listeners) {
                l.onOptionsReady(engineOptions);
            }
        } else if (line.startsWith(Uci.OPTION)) {
            try {
                EngineOption opt = EngineOptionFactory.getEngineOption(line);
                if (opt != null) {
                    engineOptions.add(opt);
                }
            } catch (Exception e) {
                Log.e(TAG, "Cannot parse option: " + line);
                e.printStackTrace();
            }
        }
    }

    public List<EngineOption> getEngineOptions() {
        return engineOptions;
    }

    public void setEngineOptions(List<EngineOption> engineOptions) {
        this.engineOptions = engineOptions;
    }

    public String getEngineName() {
        return engineName;
    }

    public void setEngineName(String engineName) {
        setReady(false);
        engineOptions.clear();
        this.engineName = engineName;
    }

    public boolean isReady() {
        return ready;
    }

    public void setReady(boolean ready) {
        this.ready = ready;
    }

    public void addListener(EngineEventListener listener) {
        listeners.add(listener);
    }

    public void removeListener(EngineEventListener listener) {
        listeners.remove(listener);
    }

    public interface EngineEventListener {

        void onOptionsReady(List<EngineOption> engineOptions);

    }
}
