package chess.mobile.engineoptions;

import android.support.v7.widget.RecyclerView;
import android.util.SparseArray;
import android.view.LayoutInflater;
import android.view.ViewGroup;

import java.util.List;

import chess.mobile.R;
import chess.mobile.engineoptions.holder.EngineOptionViewHolder;
import chess.mobile.engineoptions.holder.OptionButtonViewHolder;
import chess.mobile.engineoptions.holder.OptionCheckViewHolder;
import chess.mobile.engineoptions.holder.OptionComboViewHolder;
import chess.mobile.engineoptions.holder.OptionSpinViewHolder;
import chess.mobile.engineoptions.holder.OptionTextViewHolder;
import chess.mobile.model.engineoptions.EngineOption;
import chess.mobile.util.OnItemClickListener;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineOptionsAdapter extends RecyclerView.Adapter<EngineOptionViewHolder> {
    private SparseArray<ViewHolderProducer> viewHolderProducersMap;

    private List<EngineOption> dataSet;
    private OnItemClickListener<EngineOption> onItemClickListener;


    public EngineOptionsAdapter(List<EngineOption> dataSet, OnItemClickListener<EngineOption> onItemClickListener) {
        this.dataSet = dataSet;
        this.onItemClickListener = onItemClickListener;
        viewHolderProducersMap = new SparseArray<>();
        viewHolderProducersMap.put(EngineOption.TYPE_CHECK,
                (i, p) -> new OptionCheckViewHolder(i.inflate(R.layout.item_option_check, p, false)));
        viewHolderProducersMap.put(EngineOption.TYPE_BUTTON,
                (i, p)-> new OptionButtonViewHolder(i.inflate(R.layout.item_option_button, p, false)));
        viewHolderProducersMap.put(EngineOption.TYPE_SPIN,
                (i, p)-> new OptionSpinViewHolder(i.inflate(R.layout.item_option_spin, p, false)));
        viewHolderProducersMap.put(EngineOption.TYPE_COMBO,
                (i, p)-> new OptionComboViewHolder(i.inflate(R.layout.item_option_combo, p, false)));
        viewHolderProducersMap.put(EngineOption.TYPE_STRING,
                (i, p)-> new OptionTextViewHolder(i.inflate(R.layout.item_option_string, p, false)));
    }

    @Override
    public EngineOptionViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        LayoutInflater inflater = LayoutInflater.from(parent.getContext());
        return viewHolderProducersMap.get(viewType).createViewHolder(inflater, parent);
    }

    @Override
    public int getItemCount() {
        return dataSet.size();
    }

    @Override
    public int getItemViewType(int position) {
        return dataSet.get(position).getType();
    }

    @Override
    public void onBindViewHolder(EngineOptionViewHolder holder, int position) {
        holder.bind(dataSet.get(holder.getAdapterPosition()), onItemClickListener);
    }


    interface ViewHolderProducer {
        EngineOptionViewHolder createViewHolder(LayoutInflater inflater, ViewGroup parent);
    }
}
