package chess.mobile.engineoptions;

import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.BaseTransientBottomBar;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.model.engineoptions.EngineOption;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineOptionsFragment extends Fragment implements EngineOptionsContract.View {

    @BindView(R.id.recycler_view) RecyclerView recyclerView;
    private EngineOptionsAdapter adapter;
    private List<EngineOption> engineOptions = new ArrayList<>();
    private EngineOptionsContract.Presenter presenter;

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
    }

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = LayoutInflater.from(getActivity()).inflate(R.layout.fragment_engine_options, null);

        ButterKnife.bind(this, view);

        recyclerView.setLayoutManager(new LinearLayoutManager(getActivity()));
        recyclerView.setAdapter(adapter = new EngineOptionsAdapter(engineOptions, presenter));
        return view;
    }

    @Override
    public void setPresenter(EngineOptionsContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @Override
    public void showOptions(List<EngineOption> engineOptions) {
        this.engineOptions.clear();
        this.engineOptions.addAll(engineOptions);
        adapter.notifyDataSetChanged();
    }

    @Override
    public void showOptionExecuted(EngineOption object) {
        showMessage(object.getName() + " " + getString(R.string.executed));
    }

    @Override
    public void showOptionsSaved() {
        showMessage(getString(R.string.options_saved));
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        inflater.inflate(R.menu.options, menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == R.id.save) {
            presenter.save();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void showMessage(String msg) {
        if (getView() != null) {
            Snackbar.make(getView(), msg, BaseTransientBottomBar.LENGTH_LONG).show();
        }
    }
}
