package chess.mobile.engineoptions;

import android.view.View;

import org.greenrobot.eventbus.EventBus;

import java.util.List;

import chess.mobile.dao.UserSettingsDAO;
import chess.mobile.event.ConnectToWebSocketEvent;
import chess.mobile.event.DisconnectFromWebSocketEvent;
import chess.mobile.event.SendMessageEvent;
import chess.mobile.dao.manager.EngineManager;
import chess.mobile.model.engineoptions.EngineOption;
import chess.mobile.model.engineoptions.EngineOptionButton;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineOptionsPresenter implements EngineOptionsContract.Presenter {

    private EngineOptionsContract.View view;
    private UserSettingsDAO userSettingsDAO;

    public EngineOptionsPresenter(EngineOptionsContract.View view, UserSettingsDAO userSettingsDAO) {
        this.view = view;
        this.userSettingsDAO = userSettingsDAO;
        view.setPresenter(this);
    }

    @Override
    public void start() {
        view.showOptions(EngineManager.getInstance().getEngineOptions());
        EventBus.getDefault().post(new ConnectToWebSocketEvent(userSettingsDAO.getUrl().replace("http", "ws") + "/ws_engine"));
    }


    @Override
    public void stop() {
        EventBus.getDefault().post(new DisconnectFromWebSocketEvent());
    }

    @Override
    public void onItemClick(View v, int position, EngineOption object) {
        if (object instanceof EngineOptionButton) {
            EventBus.getDefault().post(new SendMessageEvent(object.asSetOptionUciString()));
            view.showOptionExecuted(object);
        }
    }

    @Override
    public void save() {
        List<EngineOption> options = EngineManager.getInstance().getEngineOptions();
        for (EngineOption o : options) {
            if (o instanceof EngineOptionButton) continue;
            if (o.isValid() && o.hasChanged()) {
                o.save();
                EventBus.getDefault().post(new SendMessageEvent(o.asSetOptionUciString()));
            } else {
                o.restore();
            }
        }
        view.showOptionsSaved();
    }
}
