package chess.mobile.engineoutput;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.annotation.StringRes;
import android.support.design.widget.BaseTransientBottomBar;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.text.Html;
import android.text.method.ScrollingMovementMethod;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ProgressBar;
import android.widget.TextView;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import chess.mobile.R;
import chess.mobile.engineoptions.EngineOptionsActivity;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineOutputFragment extends Fragment implements EngineOutputContract.View {

    public static final int REQUEST_SELECT_ENGINE = 364;

    @BindView(R.id.engine_name) TextView engineName;
    @BindView(R.id.output) TextView output;
    @BindView(R.id.status) TextView status;
    @BindView(R.id.progress) ProgressBar progressBar;
    @BindView(R.id.rating) TextView rating;

    private EngineOutputContract.Presenter presenter;

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
    }

    @Override
    public void onPause() {
        super.onPause();
        presenter.stop();
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_engine_output, container, false);
        ButterKnife.bind(this, view);
        output.setMovementMethod(new ScrollingMovementMethod());
        output.setText("");
        return view;
    }

    @Override
    public void setEngineName(String engineName) {
        this.engineName.setText(engineName);
        rating.setText("");
    }

    @Override
    @OnClick(R.id.change_engine)
    public void showEnginesList() {
        SelectEngineDialogFragment fragment = new SelectEngineDialogFragment();
        fragment.setTargetFragment(this, REQUEST_SELECT_ENGINE);
        fragment.show(getFragmentManager(), fragment.getClass().getSimpleName());
    }

    @OnClick(R.id.run)
    public void runOnClick() {
        presenter.run();
    }

    @OnClick(R.id.pause)
    public void pauseOnClick() {
        presenter.pause();
    }

    @OnClick(R.id.options)
    public void optionsOnClick() {
        presenter.showOptions();
    }

    @Override
    public void showOutput(String output) {
        getActivity().runOnUiThread(() -> {
            EngineOutputFragment.this.output.setText(Html.fromHtml(output));
        });
    }

    @Override
    public boolean isActive() {
        return getActivity() != null;
    }

    @Override
    public void showOptions() {
        Intent intent = new Intent(getActivity(), EngineOptionsActivity.class);
        startActivity(intent);
    }

    @Override
    public void showOptionsNotReadyError() {
        showError(R.string.err_options_not_ready);
    }

    @Override
    public void showNoEngineSelected() {
        hideStatusMessage();
    }

    @Override
    public void showEngineBooting() {
        showStatusMessage(getString(R.string.engine_booting), true);
    }

    @Override
    public void showEngineRunning() {
        showStatusMessage(getString(R.string.engine_running), true);
    }

    @Override
    public void showEngineIdle() {
        showStatusMessage(getString(R.string.engine_ready), false);
    }

    @Override
    public void showDepthAndEvaluation(int cp, int depth) {
        getActivity().runOnUiThread(() -> {
            String r = "[" + ((cp > 0 ? "+" : "") + getString(R.string.format_rating, cp / 100.0f)) + "] ";
            r += "d=" + depth;
            rating.setText(r.replace(',', '.'));
        });
    }

    @Override
    public void showEngineNotReadyError() {
        showError(R.string.engine_not_ready);
    }

    @Override
    public void setPresenter(EngineOutputContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == REQUEST_SELECT_ENGINE) {
            if (resultCode == Activity.RESULT_OK) {
                String engineName = data.getStringExtra(SelectEngineDialogFragment.RESULT_ENGINE_NAME);
                presenter.onEngineChanged(engineName);
            }
        }
    }

    private void showError(@StringRes int message) {
        if (getView() != null) {
            Snackbar.make(getView(), message, BaseTransientBottomBar.LENGTH_LONG).show();
        }
    }

    private void hideStatusMessage() {
        getActivity().runOnUiThread(() -> {
            status.setVisibility(View.GONE);
            progressBar.setVisibility(View.GONE);
        });
    }

    private void showStatusMessage(String msg, boolean showProgress) {
        getActivity().runOnUiThread(() -> {
            status.setVisibility(View.VISIBLE);
            status.setText(msg);
            progressBar.setVisibility(showProgress ? View.VISIBLE : View.GONE);
        });
    }
}
