package chess.mobile.engineoutput;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.BaseTransientBottomBar;
import android.support.design.widget.Snackbar;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.dao.EngineDAO;
import chess.mobile.dao.EngineDAOImpl;
import chess.mobile.dao.UserSettingsDAOImpl;
import chess.mobile.model.Engine;
import chess.mobile.rest.ResponseListener;
import chess.mobile.util.OnItemClickListener;

/**
 * Created by Stanisław Kabaciński.
 */

public class SelectEngineDialogFragment extends DialogFragment implements OnItemClickListener<Engine> {

    public static final String RESULT_ENGINE_NAME = "engine_name";

    @BindView(R.id.recycler_view) RecyclerView recyclerView;

    private List<Engine> dataSet = new ArrayList<>(0);

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        LayoutInflater inflater = LayoutInflater.from(getContext());
        View view = inflater.inflate(R.layout.fragment_engines_list, null, false);
        ButterKnife.bind(this, view);

        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        recyclerView.setAdapter(new EngineListAdapter());

        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(R.string.select_engine)
                .setView(view);
        return builder.create();
    }

    @Override
    public void onResume() {
        super.onResume();
        EngineDAO engineDAO = new EngineDAOImpl(new UserSettingsDAOImpl(getActivity()));
        engineDAO.getEnginesList(new ResponseListener<Engine[]>() {
            @Override
            public void onRequestFailure(Exception e) {
                if (getView() != null) {
                    Snackbar.make(getView(), R.string.cant_read_engines_list, BaseTransientBottomBar.LENGTH_LONG).show();
                }
                dismiss();
            }

            @Override
            public void onRequestSuccess(Engine[] response) {
                dataSet.clear();
                dataSet.addAll(Arrays.asList(response));
                recyclerView.getAdapter().notifyDataSetChanged();
            }
        });
    }

    @Override
    public void onItemClick(View v, int position, Engine object) {
        Fragment fragment  = getTargetFragment();
        Intent intent = new Intent();
        intent.putExtra(RESULT_ENGINE_NAME, object.name);
        if (fragment != null) {
            fragment.onActivityResult(getTargetRequestCode(), Activity.RESULT_OK, intent);
        }
        dismiss();
    }

    private class EngineListAdapter extends RecyclerView.Adapter<EngineViewHolder> {

        @Override
        public EngineViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            LayoutInflater inflater = LayoutInflater.from(parent.getContext());
            return new EngineViewHolder(inflater.inflate(
                    R.layout.item_engine, parent, false
            ), SelectEngineDialogFragment.this);
        }

        @Override
        public void onBindViewHolder(EngineViewHolder holder, int position) {
            holder.bind(dataSet.get(holder.getAdapterPosition()));
        }

        @Override
        public int getItemCount() {
            return dataSet.size();
        }
    }

}
