package chess.mobile.game;

import android.Manifest;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.NavigationView;
import android.support.design.widget.TabLayout;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v4.view.GravityCompat;
import android.support.v4.view.ViewPager;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBarDrawerToggle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Toast;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.browsegames.BrowseActivity;
import chess.mobile.chessgame.ChessGame;
import chess.mobile.chessgame.ChessGameImpl;
import chess.mobile.chessgame.ObservableChessGame;
import chess.mobile.chessgame.PossibleMovesProviderImpl;
import chess.mobile.chessgame.SANHelper;
import chess.mobile.chessgame.SANMoveMaker;
import chess.mobile.chessgame.pgn.PGNGame;
import chess.mobile.dao.EngineDAO;
import chess.mobile.dao.EngineDAOImpl;
import chess.mobile.dao.GameMetaDAOImpl;
import chess.mobile.dao.SavedGamesDAO;
import chess.mobile.dao.SavedGamesDAOImpl;
import chess.mobile.dao.UserSettingsDAO;
import chess.mobile.dao.UserSettingsDAOImpl;
import chess.mobile.engineoutput.EngineOutputContract;
import chess.mobile.engineoutput.EngineOutputFragment;
import chess.mobile.engineoutput.EngineOutputPresenter;
import chess.mobile.event.MessageReceivedEvent;
import chess.mobile.login.LoginActivity;
import chess.mobile.dao.manager.EngineManager;
import chess.mobile.meta.AddEditMetaActivity;
import chess.mobile.model.UciInfo;
import chess.mobile.moveslist.MovesListFragment;
import chess.mobile.moveslist.MovesListPresenter;
import chess.mobile.wsclient.WebSocketClient;
import uk.co.chrisjenx.calligraphy.CalligraphyContextWrapper;

import static butterknife.ButterKnife.findById;

public class GameActivity extends AppCompatActivity
        implements NavigationView.OnNavigationItemSelectedListener {
    public static final String KEY_GAME_ID = "game_id";
    private static final int PERMISSIONS_WRITE_EXTERNAL_STORAGE = 716;
    private static final int REQUEST_LOAD_FILE = 2134;
    private static final String TAG = GameActivity.class.getSimpleName();

    @BindView(R.id.view_pager) ViewPager pager;

    private FragmentPagerAdapter adapter;
    private ObservableChessGame chessGame;
    private UserSettingsDAO userSettingsDAO;

    private EngineOutputContract.Presenter engineOutputPresenter;
    private MovesListPresenter movesListPresenter;
    private BoardPresenter boardPresenter;
    private LoadGame loadGameTask;
    private ProgressDialog loadingDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        this.userSettingsDAO = new UserSettingsDAOImpl(this);
        setContentView(R.layout.activity_game);
        ButterKnife.bind(this);
        setupNavigation();
        setupViewPager();
        startService(new Intent(this, WebSocketClient.class));
        EventBus.getDefault().register(this);

        readGameIfAvailable(getIntent(), true);
    }

    private void readGameIfAvailable(Intent intent, boolean fallbackOnLastSavedGame) {
        if (intent != null && intent.hasExtra(KEY_GAME_ID)) {
            int id = intent.getIntExtra(KEY_GAME_ID, -1);
            loadGameTask = new LoadGame();
            loadingDialog = ProgressDialog.show(this, null, getString(R.string.please_wait), true, false);
            loadGameTask.execute(id);
        } else if (fallbackOnLastSavedGame) {
            loadingDialog = ProgressDialog.show(this, null, getString(R.string.please_wait), true, false);
            new LoadTmpGame().execute();
        }
    }

    @Override
    protected void onDestroy() {
        EventBus.getDefault().unregister(this);
        stopService(new Intent(this, WebSocketClient.class));
        if (loadingDialog != null) {
            loadingDialog.dismiss();
        }
        if (loadGameTask != null) {
            loadGameTask.cancel(true);
            loadGameTask = null;
        }
        saveGameInBackground();
        super.onDestroy();
    }

    @Subscribe
    @SuppressWarnings("unused")
    public void onMessageReceived(MessageReceivedEvent event) {
        EngineManager.getInstance().interceptEngineOutput(event.message);
        engineOutputPresenter.onMessageReceived(event.message);
        UciInfo info = UciInfo.getUciInfo(event.message, chessGame.getMoveList().size());
        if (info != null) {
            info.pv = SANHelper.getSanFromLAN(info.pv, chessGame.clone());
            engineOutputPresenter.onInfoReceived(info);
            boardPresenter.onInfoReceived(info);
        }
    }

    private void setupViewPager() {
        chessGame = new ObservableChessGame();

        adapter = new FragmentPagerAdapter(getSupportFragmentManager());
        BoardFragment boardFragment = new BoardFragment();
        boardPresenter = new BoardPresenter(boardFragment);
        boardFragment.setChessGame(chessGame);
        adapter.addFragment(boardFragment, getString(R.string.game));

        UserSettingsDAO userSettingsDAO = new UserSettingsDAOImpl(this);
        EngineDAO engineDAO = new EngineDAOImpl(userSettingsDAO);

        MovesListFragment movesListFragment = new MovesListFragment();
        movesListPresenter = new MovesListPresenter(movesListFragment, chessGame);
        adapter.addFragment(movesListFragment, getString(R.string.moves_list));

        EngineOutputFragment engineOutputFragment = new EngineOutputFragment();
        this.engineOutputPresenter = new EngineOutputPresenter(engineOutputFragment, engineDAO, chessGame, userSettingsDAO);
        adapter.addFragment(engineOutputFragment, getString(R.string.engine));
        pager.setOffscreenPageLimit(2);
        pager.setAdapter(adapter);
        TabLayout tabLayout = findById(this, R.id.tab_layout);
        tabLayout.setupWithViewPager(pager);
    }

    private void setupNavigation() {
        Toolbar toolbar = findById(this, R.id.toolbar);
        setSupportActionBar(toolbar);

        DrawerLayout drawer = findById(this, R.id.drawer_layout);
        ActionBarDrawerToggle toggle = new ActionBarDrawerToggle(
                this, drawer, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close);
        drawer.addDrawerListener(toggle);
        toggle.syncState();

        NavigationView navigationView = findById(this, R.id.nav_view);
        navigationView.setNavigationItemSelectedListener(this);
        navigationView.setCheckedItem(R.id.nav_game);
    }

    @Override
    public void onBackPressed() {
        DrawerLayout drawer = findById(this, R.id.drawer_layout);
        if (drawer.isDrawerOpen(GravityCompat.START)) {
            drawer.closeDrawer(GravityCompat.START);
        } else {
            super.onBackPressed();
        }
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.chess_game, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.action_save:
                if (ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)
                        != PackageManager.PERMISSION_GRANTED) {
                    ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                            PERMISSIONS_WRITE_EXTERNAL_STORAGE);
                } else {
                    saveGame();
                }
                return true;
            case R.id.action_edit_meta:
                Intent intent = new Intent(this, AddEditMetaActivity.class);
                startActivity(intent);
                return true;
            case R.id.action_next:
                movesListPresenter.selectNextMove();
                return true;
            case R.id.action_previous:
                movesListPresenter.selectPreviousMove();
                return true;
            case R.id.action_new_game:
                showNewGameDialog();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    private void showNewGameDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.dialog_new_game_title)
                .setMessage(R.string.dialog_new_game_message)
                .setPositiveButton(R.string.dialog_new_game_positive, (dialog, i) -> {
                    chessGame.reset();
                    GameMetaDAOImpl.getInstance().saveMeta(new ArrayList<>());
                })
                .setNegativeButton(R.string.dialog_new_game_negative, (dialog, i) -> dialog.dismiss())
                .show();
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode) {
            case PERMISSIONS_WRITE_EXTERNAL_STORAGE: {
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    saveGame();
                } else {
                    Toast.makeText(this, R.string.cannot_read_saved_games, Toast.LENGTH_LONG).show();
                    this.finish();
                }
            }
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == RESULT_OK && requestCode == REQUEST_LOAD_FILE) {
            readGameIfAvailable(data, false);
        }
    }

    private void saveGameInBackground() {
        this.saveGame(".tmp", true);
        this.userSettingsDAO.setLastSelectedIndex(movesListPresenter.getSelectedIndex());
    }

    private void saveGame() {
        SimpleDateFormat format = new SimpleDateFormat("yyyy.MM.dd_HH:mm:ss", Locale.US);
        this.saveGame(format.format(new Date()), false);
    }

    private void saveGame(String name, boolean inBackground) {
        SavedGamesDAO savedGamesDAO = SavedGamesDAOImpl.getInstance();
        PGNGame pgnGame = new PGNGame();
        pgnGame.setMeta(GameMetaDAOImpl.getInstance().getMeta());
        pgnGame.setEntities(movesListPresenter.getMovesList());

        if (!inBackground) {
            loadingDialog = new ProgressDialog(this);
            loadingDialog.setTitle(R.string.saving);
            loadingDialog.setMessage(getString(R.string.please_wait));
            loadingDialog.show();
        }

        savedGamesDAO.saveChessGame(name, pgnGame, new SavedGamesDAO.Callback<String>() {
            @Override
            public void onSuccess(String fileName) {
                if (loadingDialog != null) {
                    Toast.makeText(GameActivity.this, getString(R.string.game_saved), Toast.LENGTH_LONG).show();
                    loadingDialog.dismiss();
                }
            }

            @Override
            public void onFailure(Exception e) {
                if (loadingDialog != null) {
                    Toast.makeText(GameActivity.this, R.string.cannot_save_game, Toast.LENGTH_LONG).show();
                    loadingDialog.dismiss();
                }
            }
        });
    }

    @SuppressWarnings("StatementWithEmptyBody")
    @Override
    public boolean onNavigationItemSelected(@NonNull MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.nav_saved_games) {
            Intent intent = new Intent(this, BrowseActivity.class);
            startActivityForResult(intent, REQUEST_LOAD_FILE);
        } else if (id == R.id.nav_login) {
            Intent intent = new Intent(this, LoginActivity.class);
            startActivity(intent);
        }

        DrawerLayout drawer = findById(this, R.id.drawer_layout);
        drawer.closeDrawer(GravityCompat.START);
        return true;
    }

    private class LoadGame extends AsyncTask<Integer, Void, PGNGame> {
        ChessGame chessGame = new ChessGameImpl();

        @Override
        protected PGNGame doInBackground(Integer... integers) {
            try {
                SavedGamesDAO savedGamesDAO = SavedGamesDAOImpl.getInstance();
                long time = System.currentTimeMillis();
                PGNGame game = savedGamesDAO.getGame(integers[0]);
                Log.d(TAG, String.format("Loading PGN game: %.10fs", (time - System.currentTimeMillis()) / 1000.0));
                GameMetaDAOImpl.getInstance().saveMeta(game.getMeta());
                time = System.currentTimeMillis();
                SANMoveMaker maker = new SANMoveMaker(chessGame, new PossibleMovesProviderImpl(chessGame));
                maker.processMoves(game.getEntities());
                Log.d(TAG, String.format("Making moves: %.10fs", (time - System.currentTimeMillis()) / 1000.0));
                return game;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        @Override
        protected void onPostExecute(PGNGame pgnGame) {
            if (loadingDialog != null) {
                loadingDialog.dismiss();
            }
            if (pgnGame == null) {
                Toast.makeText(GameActivity.this, R.string.cannot_read_game, Toast.LENGTH_LONG).show();
            } else {
                GameActivity.this.chessGame.setChessGame(chessGame);
                movesListPresenter.setMovesList(pgnGame.getEntities());
                movesListPresenter.setSelectedIndex(0);
            }
        }

    }

    private class LoadTmpGame extends AsyncTask<Void, Void, PGNGame> {
        ChessGame chessGame = new ChessGameImpl();

        @Override
        protected PGNGame doInBackground(Void... v) {
            try {
                PGNGame game = SavedGamesDAOImpl.getInstance().getTemporalGame();
                GameMetaDAOImpl.getInstance().saveMeta(game.getMeta());
                SANMoveMaker maker = new SANMoveMaker(chessGame, new PossibleMovesProviderImpl(chessGame));
                maker.processMoves(game.getEntities());
                return game;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            }
        }

        @Override
        protected void onPostExecute(PGNGame pgnGame) {
            if (loadingDialog != null) {
                loadingDialog.dismiss();
            }
            if (pgnGame == null) {
                Toast.makeText(GameActivity.this, R.string.cannot_read_game, Toast.LENGTH_LONG).show();
                return;
            }
            GameActivity.this.chessGame.setChessGame(chessGame);
            movesListPresenter.setMovesList(pgnGame.getEntities());
            movesListPresenter.setSelectedIndex(userSettingsDAO.getLastSelectedIndex());
        }
    }

    @Override
    protected void attachBaseContext(Context newBase) {
        super.attachBaseContext(CalligraphyContextWrapper.wrap(newBase));
    }
}
