package chess.mobile.login;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.TextInputEditText;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import chess.mobile.R;

/**
 * Created by Stanisław Kabaciński.
 */

public class LoginFragment extends Fragment implements LoginContract.View {

    public LoginContract.Presenter presenter;

    @BindView(R.id.url) TextInputEditText url;
    @BindView(R.id.login) TextInputEditText login;
    @BindView(R.id.password) TextInputEditText password;

    private ProgressDialog progressDialog;

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View v = inflater.inflate(R.layout.login_fragment, container, false);
        ButterKnife.bind(this, v);
        return v;
    }

    @Override
    public void setPresenter(LoginContract.Presenter presenter) {
        this.presenter = presenter;
    }

    @OnClick(R.id.login_button)
    public void onClick(View v) {
        presenter.login(url.getText(), login.getText(), password.getText());
    }

    @Override
    public void setUrl(String url) {
        this.url.setText(url);
    }

    @Override
    public void setLogin(String login) {
        this.login.setText(login);
    }

    @Override
    public void showProgress(boolean show) {
        if (getActivity() == null) {
            return;
        }
        if (show) {
            if (progressDialog == null) {
                progressDialog = new ProgressDialog(getContext());
                progressDialog.setTitle(R.string.connecting);
                progressDialog.setCancelable(false);
            }
            progressDialog.show();
        } else {
            if (progressDialog != null) {
                progressDialog.hide();
            }
        }
    }

    @Override
    public void showError(boolean show) {
        if (getActivity() == null) {
            return;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(R.string.something_went_wrong)
                .setMessage(R.string.login_error)
                .setCancelable(true)
                .show();
    }

    @Override
    public void showSuccess(boolean success) {
        if (getActivity() == null) {
            return;
        }
        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(R.string.success)
                .setMessage(getString(R.string.you_are_now_connected_to, url.getText().toString()))
                .setNeutralButton(R.string.ok, (dialog, which) -> {
                    dialog.dismiss();
                    this.getActivity().finish();
                }).show();
    }
}
