package chess.mobile.login;

import chess.mobile.dao.ServerAccessDAO;
import chess.mobile.dao.UserSettingsDAO;
import chess.mobile.model.LoginForm;
import chess.mobile.model.LoginResponse;
import chess.mobile.rest.ResponseListener;
import chess.mobile.rest.ServiceGenerator;

/**
 * Created by Stanisław Kabaciński.
 */

public class LoginPresenter implements LoginContract.Presenter {

    private LoginContract.View view;
    private ServerAccessDAO serverAccessDAO;
    private UserSettingsDAO userSettingsDAO;

    public LoginPresenter(LoginContract.View view, ServerAccessDAO serverAccessDAO,
                          UserSettingsDAO userSettingsDAO) {
        this.view = view;
        this.view.setPresenter(this);
        this.serverAccessDAO = serverAccessDAO;
        this.userSettingsDAO = userSettingsDAO;
    }

    @Override
    public void start() {
        view.setUrl(userSettingsDAO.getUrl());
        view.setLogin(userSettingsDAO.getLogin());
    }

    @Override
    public void login(CharSequence url, CharSequence login, CharSequence password) {
        ServiceGenerator.setBaseUrl(url.toString());
        userSettingsDAO.setUrl(url.toString());
        userSettingsDAO.setLogin(login.toString());
        LoginForm form = new LoginForm();
        form.login = login.toString();
        form.password = password.toString();
        view.showProgress(true);

        serverAccessDAO.login(form, new ResponseListener<LoginResponse>() {
            @Override
            public void onRequestFailure(Exception e) {
                view.showProgress(false);
                view.showError(true);
            }

            @Override
            public void onRequestSuccess(LoginResponse response) {
                view.showProgress(false);
                if (response.isSuccess()) {
                    userSettingsDAO.setAuthToken(response.getToken());
                    view.showSuccess(true);
                } else {
                    view.showError(true);
                }
            }
        });
    }
}
