package chess.mobile.meta;


import android.app.Activity;
import android.app.Dialog;
import android.content.Intent;
import android.os.Bundle;
import android.support.design.widget.TextInputEditText;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.View;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;

/**
 * Created by Stanisław Kabaciński.
 */

public class AddEditMetaDialog extends DialogFragment {

    public static final String KEY_KEY = "key";
    public static final String KEY_VALUE = "value";
    public static final String KEY_POSITION = "position";
    public static final String KEY_REMOVE = "remove";
    public static final String KEY_ALLOW_REMOVE = "allow_remove";
    public static final String KEY_ALLOW_KEY_EDIT = "allow_key_edit";

    @BindView(R.id.key) TextInputEditText key;
    @BindView(R.id.value) TextInputEditText value;

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        LayoutInflater inflater = LayoutInflater.from(getContext());
        View v = inflater.inflate(R.layout.dialog_meta, null, false);
        ButterKnife.bind(this, v);

        Bundle args = getArguments();

        String key = args.getString(KEY_KEY, "");
        String value = args.getString(KEY_VALUE, "");
        int position = args.getInt(KEY_POSITION, 0);
        boolean allowRemove = args.getBoolean(KEY_ALLOW_REMOVE, true);
        boolean allowKeyEdit = args.getBoolean(KEY_ALLOW_KEY_EDIT, true);

        this.key.setText(key);
        this.value.setText(value);
        this.key.setEnabled(allowKeyEdit);

        AlertDialog.Builder builder = new AlertDialog.Builder(getContext());
        builder.setTitle(R.string.edit)
                .setView(v)
                .setPositiveButton(R.string.accept, (dialog, which) -> {
                    Intent intent = getResponseIntent(position, false);
                    getTargetFragment().onActivityResult(getTargetRequestCode(), Activity.RESULT_OK, intent);
                })
                .setNeutralButton(R.string.cancel, (dialog, which) -> {
                    dialog.dismiss();
                });
        if (allowRemove) {
            builder.setNegativeButton(R.string.remove, (dialog, which) -> {
                Intent intent = getResponseIntent(position, true);
                getTargetFragment().onActivityResult(getTargetRequestCode(), Activity.RESULT_OK, intent);
            });
        }
        return builder.create();
    }

    private Intent getResponseIntent(int position, boolean remove) {
        Intent args = new Intent();
        args.putExtra(KEY_KEY, key.getText().toString());
        args.putExtra(KEY_VALUE, value.getText().toString());
        args.putExtra(KEY_POSITION, position);
        if (remove) {
            args.putExtra(KEY_REMOVE, true);
        }
        return args;
    }
}
