package chess.mobile.meta;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import chess.mobile.R;
import chess.mobile.chessgame.pgn.Meta;

/**
 * Created by Stanisław Kabaciński.
 */

public class AddEditMetaFragment extends Fragment implements AddEditMetaContract.View {

    private static final int REQUEST_ADD_META = 457;
    private static final int REQUEST_EDIT_META = 453;

    @BindView(R.id.recycler_view) RecyclerView recyclerView;

    private AddEditMetaContract.Presenter presenter;
    private List<Meta> dataSet = new ArrayList<>();
    private Adapter adapter = new Adapter();

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setHasOptionsMenu(true);
    }

    @Override
    public void onResume() {
        super.onResume();
        presenter.start();
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        inflater.inflate(R.menu.add_edit_meta, menu);
        super.onCreateOptionsMenu(menu, inflater);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.add_meta: {
                showAddEditMetaDialog(dataSet.size(), "", "", false, true);
                return true;
            }
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View v = inflater.inflate(R.layout.fragment_add_edit_meta, container, false);
        ButterKnife.bind(this, v);

        recyclerView.setLayoutManager(new LinearLayoutManager(getContext()));
        recyclerView.setAdapter(adapter);

        return v;
    }

    @Override
    public void showMeta(List<Meta> meta) {
        this.dataSet = meta;
        adapter.notifyDataSetChanged();
    }

    @Override
    public void addMeta(Meta meta) {
        adapter.notifyItemInserted(dataSet.size());
    }

    @Override
    public void removeMeta(int position) {
        adapter.notifyItemRemoved(position);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK) {
            int position = data.getIntExtra(AddEditMetaDialog.KEY_POSITION, -1);
            String key = data.getStringExtra(AddEditMetaDialog.KEY_KEY);
            String value = data.getStringExtra(AddEditMetaDialog.KEY_VALUE);
            if (requestCode == REQUEST_EDIT_META) {
                if (data.getBooleanExtra(AddEditMetaDialog.KEY_REMOVE, false)) {
                    presenter.removeMeta(position);
                } else {
                    presenter.updateMeta(position, key, value);
                }
            } else if (requestCode == REQUEST_ADD_META) {
                presenter.addMeta(key, value);
            }
        }
    }

    @Override
    public void showAddEditMetaDialog(int position, String key, String value, boolean isRequired, boolean add) {
        AddEditMetaDialog dialog = new AddEditMetaDialog();
        Bundle args = new Bundle();
        args.putInt(AddEditMetaDialog.KEY_POSITION, position);
        args.putString(AddEditMetaDialog.KEY_KEY, key);
        args.putString(AddEditMetaDialog.KEY_VALUE, value);
        args.putBoolean(AddEditMetaDialog.KEY_ALLOW_REMOVE, !isRequired && !add);
        args.putBoolean(AddEditMetaDialog.KEY_ALLOW_KEY_EDIT, !isRequired);
        dialog.setArguments(args);
        dialog.setTargetFragment(this, add ? REQUEST_ADD_META : REQUEST_EDIT_META);
        dialog.show(getFragmentManager(), dialog.getClass().getSimpleName());
    }

    @Override
    public void updateMeta(int position) {
        adapter.notifyItemChanged(position);
    }

    @Override
    public void setPresenter(AddEditMetaContract.Presenter presenter) {
        this.presenter = presenter;
    }

    private class Adapter extends RecyclerView.Adapter<MetaViewHolder> {

        @Override
        public MetaViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
            return new MetaViewHolder(LayoutInflater.from(parent.getContext()).inflate(
                    R.layout.item_meta, parent, false
            ));
        }

        @Override
        public void onBindViewHolder(MetaViewHolder holder, int position) {
            holder.bind(dataSet.get(holder.getAdapterPosition()), presenter);
        }

        @Override
        public int getItemCount() {
            return dataSet.size();
        }
    }
}
