package chess.mobile.model.engineoptions;

import java.util.Arrays;

import chess.mobile.Uci;
import chess.mobile.util.Strings;

/**
 * Created by Stanisław Kabaciński.
 */

public class EngineOptionFactory {

    public static EngineOption getEngineOption(String option) throws Exception {
        String name = Strings.getStringBetween(option, Uci.option.NAME, Uci.option.TYPE).trim();
        String type = Strings.getStringBetween(option, Uci.option.TYPE, Uci.option.DEFAULT).trim();

        EngineOption engineOption = null;
        switch (type) {
            case Uci.option.type.BUTTON: {
                engineOption = new EngineOptionButton(name);
                break;
            }
            case Uci.option.type.CHECK: {
                boolean value = Boolean.valueOf(Strings.getStringBetween(option, Uci.option.DEFAULT, null).trim());
                engineOption = new EngineOptionCheck(value, name);
                break;
            }
            case Uci.option.type.COMBO: {
                String value = Strings.getStringBetween(option, Uci.option.DEFAULT, Uci.option.VAR).trim();
                String[] vars = Strings.getStringBetween(option, Uci.option.VAR, null).split(Uci.option.VAR);
                for (int i = 0; i < vars.length; i++) {
                    vars[i] = vars[i].trim();
                }
                engineOption = new EngineOptionCombo(Arrays.asList(vars), value, name);
                break;
            }
            case Uci.option.type.SPIN: {
                int value = Integer.parseInt(Strings.getStringBetween(option, Uci.option.DEFAULT, Uci.option.MIN).trim());
                int min = Integer.parseInt(Strings.getStringBetween(option, Uci.option.MIN, Uci.option.MAX).trim());
                int max = Integer.parseInt(Strings.getStringBetween(option, Uci.option.MAX, null).trim());
                engineOption = new EngineOptionSpin(value, max, min, name);
                break;
            }
            case Uci.option.type.STRING: {
                String value = Strings.getStringBetween(option, Uci.option.DEFAULT, null).trim();
                engineOption = new EngineOptionString(name, value);
                break;
            }
            default:
                return null;
        }

        return engineOption;
    }
}
