package chess.mobile.moveslist;

import java.util.List;
import java.util.Observer;

import chess.mobile.BasePresenter;
import chess.mobile.BaseView;
import chess.mobile.chessgame.Comment;
import chess.mobile.chessgame.Entity;
import chess.mobile.chessgame.Move;
import chess.mobile.chessgame.NumericAnnotationGlyph;
import chess.mobile.util.OnItemClickListener;

/**
 * Created by Stanisław Kabaciński.
 */

public interface MovesListContract {

    interface View extends BaseView<Presenter> {

        void setMovesList(List<Entity> movesList);

        void addNewMove(Entity entity);

        void undoLastMove();

        void showDetailsDialog(Entity entity, int moveNo);

        void updateMove(int position);

        void notifyItemInserted(int position);

        void notifyItemRemoved(int position);

        void notifyItemChanged(int position);

        int getSelectedIndex();

        void clearSelection();

        void setSelectedIndex(int selectedIndex);

        void showEditComment(int position, Comment comment);

        void showAddComment(int position);

        void showEditGlyph(int position, int[] glyphs);

        void notifyDataSetChanged();
    }

    interface Presenter extends BasePresenter, Observer {

        void stop();

        void onCommentAdded(String comment, int position);

        void onCommentEdited(String value, int position);

        void removeComment(int position);

        void selectNextMove();

        void selectPreviousMove();

        void onMoveSelected(int position);

        boolean canDeleteMove(int position);

        void removeSubVariant(int position, Entity object);

        void removeMove(int position);

        void onSelectGlyphClicked(int position);

        void onGlyphChanged(int position, int[] nags);

        void onMoveClicked(int position);

        void onCommentClicked(int position, Comment comment);

        List<Entity> getMovesList();

        void setMovesList(List<Entity> e);

        int getSelectedIndex();

        void setSelectedIndex(int index);
    }
}
