package chess.mobile.rest;

import android.text.TextUtils;

import com.fasterxml.jackson.databind.ObjectMapper;

import java.text.SimpleDateFormat;
import java.util.Locale;

import okhttp3.OkHttpClient;
import okhttp3.logging.HttpLoggingInterceptor;
import retrofit2.Retrofit;
import retrofit2.converter.jackson.JacksonConverterFactory;

/**
 * Created 07.02.2017.
 */

public class ServiceGenerator {

    private static String baseUrl = "http://192.168.1.123:8080";

    private static Retrofit retrofit;

    private static Retrofit.Builder builder = null;

    private static HttpLoggingInterceptor logging = new HttpLoggingInterceptor()
            .setLevel(HttpLoggingInterceptor.Level.BODY);

    private static OkHttpClient.Builder httpClient = new OkHttpClient.Builder();

    private static boolean baseUrlChanged = true;

    private static void setupBuilder() {
        if (builder == null || baseUrlChanged) {
            ObjectMapper objectMapper = new ObjectMapper()
                    .setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US));
            JacksonConverterFactory jacksonConverterFactory = JacksonConverterFactory
                    .create(objectMapper);
            builder = new Retrofit.Builder()
                    .baseUrl(baseUrl)
                    .addConverterFactory(jacksonConverterFactory);
        }
    }

    public synchronized static <S> S createService(Class<S> serviceClass) {
        setupBuilder();
        if (!httpClient.interceptors().contains(logging) || baseUrlChanged) {
            httpClient.addInterceptor(logging);
            builder.client(httpClient.build());
            retrofit = builder.build();
        }

        return retrofit.create(serviceClass);
    }

    public synchronized static <S> S createService(Class<S> serviceClass, String authToken) {
        setupBuilder();
        if (!TextUtils.isEmpty(authToken)) {
            AuthenticationInterceptor interceptor =
                    new AuthenticationInterceptor(authToken);

            if (!httpClient.interceptors().contains(interceptor) || baseUrlChanged) {
                httpClient.addInterceptor(interceptor);

                if (!httpClient.interceptors().contains(logging)) {
                    httpClient.addInterceptor(logging);
                }

                builder.client(httpClient.build());
                retrofit = builder.build();
            }
            return retrofit.create(serviceClass);
        } else {
            return createService(serviceClass);
        }
    }

    public static String getBaseUrl() {
        return baseUrl;
    }

    public static void setBaseUrl(String baseUrl) {
        ServiceGenerator.builder = null;
        baseUrlChanged = true;
        ServiceGenerator.baseUrl = baseUrl;
    }
}
