package chess.mobile.view.chess_view;

import android.view.MotionEvent;
import android.view.View;

import java.util.List;

import chess.mobile.chessgame.PossibleMovesProvider;
import chess.mobile.view.chess_view.renderers.ChessboardRenderer;
import chess.mobile.view.chess_view.renderers.OverlayRenderer;

/**
 * Created by Stanisław Kabaciński.
 */

public class ChessboardController implements View.OnTouchListener {
    public static final String TAG = "ChessboardController";

    private ChessboardRenderer renderer;
    private PossibleMovesProvider movesProvider;

    private int selectionX;
    private int selectionY;
    private List<Integer> possibleMoves;
    private OnMoveListener onMoveListener;

    public ChessboardController(ChessboardRenderer renderer) {
        this.renderer = renderer;
    }

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        if (event.getAction() == MotionEvent.ACTION_UP) {
            handleTouchEvent(event.getX(), event.getY());
            return true;
        }
        return false;
    }

    private void handleTouchEvent(float tx, float ty) {
        int direction = renderer.isInverted() ? -1 : 1;
        int offset = renderer.isInverted() ? 1 : 0;
        int[][] board = renderer.getBoard();
        movesProvider.setBoard(board);

        OverlayRenderer overlay = renderer.getOverlayRenderer();

        tx = (tx - renderer.getCenterX()) * direction + 4 * renderer.getFieldSize();
        ty = (ty - renderer.getCenterY()) * direction + 4 * renderer.getFieldSize();

        int bx = (int) (tx / renderer.getFieldSize());
        int by = 7 - (int) (ty / renderer.getFieldSize());

        overlay.setShowSelection(false);
        overlay.setShowPossibleMoves(false);

        if (possibleMoves != null && isMovePossible(bx, by)) {
            if (onMoveListener != null) {
                onMoveListener.moveMade(selectionX, selectionY, bx, by);
            }
        }

        if (bx < 0 || by < 0 || bx > 7 || by > 7 || board[bx][by] == 0) {
            this.possibleMoves = null;
            return;
        }

        this.selectionX = bx;
        this.selectionY = by;
        overlay.setShowSelection(true);
        overlay.setSelection(bx, by);

        this.possibleMoves = movesProvider.getPossibleMoves(board[bx][by], bx, by);
        overlay.setPossibleMoves(this.possibleMoves);
        overlay.setShowPossibleMoves(true);
    }

    public void resetSelection() {
        if (renderer.getOverlayRenderer() != null) {
            renderer.getOverlayRenderer().setShowPossibleMoves(false);
            renderer.getOverlayRenderer().setShowSelection(false);
        }
        this.possibleMoves = null;
    }

    private boolean isMovePossible(int x, int y) {
        for (int i = 0, ii = possibleMoves.size() / 2; i < ii; i++) {
            if (x == possibleMoves.get(i * 2) && y == possibleMoves.get(i * 2 + 1)) return true;
        }
        return false;
    }

    public void setMovesProvider(PossibleMovesProvider movesProvider) {
        this.movesProvider = movesProvider;
    }

    public void setOnMoveListener(OnMoveListener onMoveListener) {
        this.onMoveListener = onMoveListener;
    }

}
