package chess.mobile.view.chess_view;

import android.content.Context;
import android.util.AttributeSet;
import android.view.SurfaceView;

import chess.mobile.chessgame.PossibleMovesProvider;
import chess.mobile.view.chess_view.renderers.ChessboardRenderer;

/**
 * Created by Stanisław Kabaciński.
 */

public class ChessboardView extends SurfaceView {
    public static final String TAG = "Chessboard View";

    private ChessboardRenderer chessboardRenderer;
    private ChessboardController chessboardController;
    private boolean inverted;

    public ChessboardView(Context context) {
        super(context);
        init();
    }

    public ChessboardView(Context context, AttributeSet attrs) {
        super(context, attrs);
        init();
    }

    public ChessboardView(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
        this.setClickable(true);
        this.setWillNotDraw(false);
        this.chessboardRenderer = new ChessboardRenderer(getContext());
        this.chessboardController = new ChessboardController(chessboardRenderer);
        setOnTouchListener(chessboardController);
        getHolder().addCallback(this.chessboardRenderer);
        setInverted(false);
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {

        int desiredWidth = 1080;
        int desiredHeight = 1080;

        int widthMode = MeasureSpec.getMode(widthMeasureSpec);
        int widthSize = MeasureSpec.getSize(widthMeasureSpec);
        int heightMode = MeasureSpec.getMode(heightMeasureSpec);
        int heightSize = MeasureSpec.getSize(heightMeasureSpec);

        int width;
        int height;

        if (widthMode == MeasureSpec.EXACTLY) {
            width = widthSize;
        } else if (widthMode == MeasureSpec.AT_MOST) {
            width = Math.min(desiredWidth, widthSize);
        } else {
            width = desiredWidth;
        }

        if (heightMode == MeasureSpec.EXACTLY) {
            height = heightSize;
        } else if (heightMode == MeasureSpec.AT_MOST) {
            height = Math.min(desiredHeight, heightSize);
        } else {
            height = desiredHeight;
        }

        if (width > height) {
            width = height;
        } else {
            height = width;
        }

        setMeasuredDimension(width, height);
    }

    public void setBoard(int[][] board) {
        this.chessboardRenderer.setBoard(board);
    }

    public void setOnMoveListener(OnMoveListener onMoveListener) {
        this.chessboardController.setOnMoveListener(onMoveListener);
    }

    public void setPossibleMovesProvider(PossibleMovesProvider possibleMovesProvider) {
        this.chessboardController.setMovesProvider(possibleMovesProvider);
    }

    public void resetSelection() {
        this.chessboardController.resetSelection();
    }

    public boolean isInverted() {
        return inverted;
    }

    public void setInverted(boolean inverted) {
        this.inverted = inverted;
        chessboardRenderer.setInverted(inverted);
    }
}
