package chess.mobile.view.chess_view.renderers;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;

/**
 * Created by Stanisław Kabaciński.
 */

public class BoardRenderer {
    public static final int FIELDS_COUNT = 8;
    public static final int BLACK = Color.rgb(113, 136, 153);
    public static final int WHITE = Color.WHITE;

    private ChessboardRenderer chessboardRenderer;
    private float fieldSize;
    private float centerX;
    private float centerY;

    private Paint paint;

    public BoardRenderer(ChessboardRenderer chessboardRenderer) {
        this.chessboardRenderer = chessboardRenderer;
        this.paint = new Paint();
        this.paint.setStyle(Paint.Style.FILL);
        this.paint.setAntiAlias(true);

        updateDimensions();
    }

    public void draw(Canvas canvas) {
        for (int x = -FIELDS_COUNT / 2, xx = FIELDS_COUNT / 2; x < xx; x++) {
            for (int y = -FIELDS_COUNT / 2, yy = FIELDS_COUNT / 2; y < yy; y++) {
                paint.setColor((x + y) % 2 == 0 ? WHITE : BLACK);
                canvas.drawRect(
                        x * fieldSize + centerX, y * fieldSize + centerY,
                        (x + 1) * fieldSize + centerX, (y + 1) * fieldSize + centerY,
                        paint);
            }
        }
    }

    void updateDimensions() {
        float x = chessboardRenderer.getWidth();
        float y = chessboardRenderer.getHeight();
        this.fieldSize = (x / y < 1 ? x : y) / FIELDS_COUNT;
        this.centerX = x / 2;
        this.centerY = y / 2;
    }

    public float getFieldSize() {
        return fieldSize;
    }

    public float getCenterX() {
        return centerX;
    }

    public float getCenterY() {
        return centerY;
    }
}
