package chess.mobile.view.chess_view.renderers;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Stanisław Kabaciński.
 */

public class OverlayRenderer {
    public static final int SELECTION_COLOR = Color.rgb(15, 155, 255);
    public static final int MOVE_COLOR = Color.argb(80, 15, 155, 255);

    private BoardRenderer boardRenderer;
    private ChessboardRenderer chessboardRenderer;
    private Paint strokePaint;
    private Paint fillPaint;

    private boolean showSelection = true;
    private boolean showPossibleMoves = true;

    private int selectionX;
    private int selectionY;

    private List<Integer> possibleMoves;


    public OverlayRenderer(BoardRenderer boardRenderer, ChessboardRenderer chessboardRenderer) {
        this.boardRenderer = boardRenderer;
        this.chessboardRenderer = chessboardRenderer;

        this.strokePaint = new Paint();
        this.strokePaint.setAntiAlias(true);
        this.strokePaint.setStyle(Paint.Style.STROKE);
        this.strokePaint.setStrokeWidth(5);
        this.strokePaint.setColor(SELECTION_COLOR);

        this.fillPaint = new Paint();
        this.fillPaint.setAntiAlias(true);
        this.fillPaint.setStyle(Paint.Style.FILL);
        this.fillPaint.setColor(MOVE_COLOR);

        this.selectionX = 2;
        this.selectionY = 2;
        this.possibleMoves = new ArrayList<>();
    }

    public void draw(Canvas canvas) {
        if (showSelection) drawSelection(canvas);
        if (showPossibleMoves) drawPossibleMoves(canvas);
    }

    private void drawPossibleMoves(Canvas canvas) {
        int direction = chessboardRenderer.isInverted() ? -1 : 1;
        int offset = chessboardRenderer.isInverted() ? 1 : 0;
        synchronized (this) {
            if (this.possibleMoves == null) return;
            for (int i = 0, ii = possibleMoves.size() / 2; i < ii; i++) {
                int bx = possibleMoves.get(i * 2);
                int by = possibleMoves.get(i * 2 + 1);
                float x = direction * (bx - 4 + offset) * boardRenderer.getFieldSize() + boardRenderer.getCenterX();
                float y = direction * (3 - by + offset) * boardRenderer.getFieldSize() + boardRenderer.getCenterY();
                canvas.drawRect(x, y,
                        x + boardRenderer.getFieldSize(), y + boardRenderer.getFieldSize(),
                        fillPaint);
            }
        }
    }

    private void drawSelection(Canvas canvas) {
        int direction = chessboardRenderer.isInverted() ? -1 : 1;
        int offset = chessboardRenderer.isInverted() ? 1 : 0;
        float x = direction * (selectionX - 4 + offset) * boardRenderer.getFieldSize() + boardRenderer.getCenterX();
        float y = direction * (3 - selectionY + offset) * boardRenderer.getFieldSize() + boardRenderer.getCenterY();
        canvas.drawRect(x, y,
                x + boardRenderer.getFieldSize(), y + boardRenderer.getFieldSize(),
                strokePaint);
    }

    public void setSelection(int x, int y) {
        this.selectionX = x;
        this.selectionY = y;
    }

    public void setPossibleMoves(List<Integer> possibleMoves) {
        synchronized (this) {
            this.possibleMoves = possibleMoves;
        }
    }

    public boolean isShowSelection() {
        return showSelection;
    }

    public void setShowSelection(boolean showSelection) {
        this.showSelection = showSelection;
    }

    public boolean isShowPossibleMoves() {
        return showPossibleMoves;
    }

    public void setShowPossibleMoves(boolean showPossibleMoves) {
        this.showPossibleMoves = showPossibleMoves;
    }
}
