package chess.mobile.view.chess_view.renderers;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;

import chess.mobile.view.chess_view.providers.PieceBitmapProvider;

/**
 * Created by Stanisław Kabaciński.
 */
public class PieceRenderer {
    private static final int FIELDS_COUNT = 8;

    private PieceBitmapProvider provider;
    private Paint paint;
    private BoardRenderer boardRenderer;
    private ChessboardRenderer chessboardRenderer;
    private int[][] board;

    public PieceRenderer(PieceBitmapProvider provider, BoardRenderer boardRenderer, ChessboardRenderer chessboardRenderer) {
        this.provider = provider;
        this.boardRenderer = boardRenderer;
        this.chessboardRenderer = chessboardRenderer;
        this.paint = new Paint();
        this.paint.setAntiAlias(true);
    }

    public void draw(Canvas canvas) {
        if (board == null) return;
        int direction = chessboardRenderer.isInverted() ? -1 : 1;
        int offset = chessboardRenderer.isInverted() ? 1 : 0;
        for (int x = -FIELDS_COUNT / 2, xx = FIELDS_COUNT / 2; x < xx; x++) {
            for (int y = -FIELDS_COUNT / 2, yy = FIELDS_COUNT / 2; y < yy; y++) {
                int i = x + FIELDS_COUNT / 2;
                int j = FIELDS_COUNT - (y + FIELDS_COUNT / 2) - 1;
                if (board[i][j] == 0) continue;
                Bitmap bitmap = provider.getBitmapForPiece(board[i][j]);
                if (bitmap == null) continue;
                canvas.drawBitmap(bitmap,
                        direction * (x + offset) * boardRenderer.getFieldSize() + boardRenderer.getCenterX(),
                        direction * (y + offset) * boardRenderer.getFieldSize() + boardRenderer.getCenterY(),
                        paint);
            }
        }
    }

    public void setBoard(int[][] board) {
        this.board = board;
    }
}
