package chess.mobile.browsegames;

import org.junit.Test;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;

import java.util.Collections;
import java.util.List;

import chess.mobile.dao.SavedGamesDAO;
import chess.mobile.model.SavedGameFileMeta;

import static org.mockito.Mockito.any;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.ArgumentMatchers.*;

/**
 * Created 27.09.2017.
 */
public class BrowsePresenterTest {
    @Test
    public void loadPGNFilesListOnStart() throws Exception {
        BrowseGamesContract.View view = mock(BrowseGamesContract.View.class);
        SavedGamesDAO dao = mock(SavedGamesDAO.class);
        BrowsePresenter presenter = new BrowsePresenter(view, dao);
        presenter.start();
        verify(dao).getPgnFilesList(any());
    }

    @Test
    public void onGameSelectedOpenProperFile() throws Exception {
        SavedGameFileMeta meta = new SavedGameFileMeta();
        meta.setPath("test");

        BrowseGamesContract.View view = mock(BrowseGamesContract.View.class);
        SavedGamesDAO dao = mock(SavedGamesDAO.class);
        BrowsePresenter presenter = new BrowsePresenter(view, dao);
        presenter.onGameSelected(meta);

        verify(view).openFile("test");
    }

    @Test
    public void removeFile() throws Exception {
        SavedGameFileMeta meta = new SavedGameFileMeta();
        meta.setPath("test");
        BrowseGamesContract.View view = mock(BrowseGamesContract.View.class);
        SavedGamesDAO dao = mock(SavedGamesDAO.class);
        doAnswer(new Answer<Void>() {
                     @Override
                     public Void answer(InvocationOnMock invocation) throws Throwable {
                         SavedGamesDAO.Callback < List < SavedGameFileMeta >> callback = invocation.getArgument(0);
                         callback.onSuccess(Collections.singletonList(meta));
                         return null;
                     }
                 }).when(dao).getPgnFilesList(any());

        BrowsePresenter presenter = new BrowsePresenter(view, dao);
        presenter.start();
        presenter.removeFile(0);
        verify(dao).removeFile(eq("test"), any());
    }

}